#include "actor.h"
#include "script.h"
#include "map.h"
#include "obj.h"
#include "gen.h"
#include "lib.h"
#include <string>
using namespace std;

const string SCRIPTFILE ( "script.txt" );

Actor::Actor ( string actorDir, int x, int y )
: stats ( actorDir + SCRIPTFILE )
, scripts ( this ) {
	mapObj = NULL;
	talkedTo = false;

	const char SCRIPT_HEADER = '^';
	const string SCRIPT_NAME ( "^Name" );
	const string SCRIPT_IMG ( "^Img" );
	const string SCRIPT_AI ( "^Ai" );
	const string SCRIPT_IMG_FRONT ( "^Front" );
	const string SCRIPT_IMG_BACK ( "^Back" );
	const string SCRIPT_IMG_RIGHT ( "^Right" );
	const string SCRIPT_IMG_LEFT ( "^Left" );
	const string SCRIPT_IMG_NIL ( "^Nil" );
	const string SCRIPT_STATS ( "^Stats" );
	const string SCRIPT_SCRIPTS ( "^Scripts" );

	FileIn actorIn ( actorDir + SCRIPTFILE );

	string line;
	while ( ! actorIn . eof () ) {
		line = actorIn . getLine ();
		if ( line == SCRIPT_NAME ) {
			name = actorIn . getLine ();
			break;
		}
	}

	if ( name . empty () )
		fatErrMsg ( "Actor's name not provided in script file \"" + actorDir + SCRIPTFILE + "\"." );

	actorIn . reopen ( actorDir + SCRIPTFILE );

	line = actorIn . getLine ();
	while ( ! actorIn . eof () ) {
		if ( line == SCRIPT_NAME ) {
			actorIn . getLine ();
			line = actorIn . getLine ();
		}

		else if ( line == SCRIPT_AI ) {
			line = actorIn . getLine ();
			string origLine = line;
			ai . att = atoi ( line . substr ( 0, line . find ( ' ' ) ) . c_str () );
			line = line . substr ( line . find ( ' ' ) + 1, string::npos );
			ai . def = atoi ( line . substr ( 0, line . find ( ' ' ) ) . c_str () );
			line = line . substr ( line . find ( ' ' ) + 1, string::npos );
			ai . wild = atoi ( line . substr ( 0, line . find ( ' ' ) ) . c_str () );

			if ( ( ai . att + ai . def + ai . wild ) != 30 )
				fatErrMsg ( "Improper combat ai values: \"" + origLine + "\"." );

			line = actorIn . getLine ();
			if ( line [ 0 ] == ':' ) {
				ai . worth = atoi ( line . substr ( 1, string::npos ) . c_str () );
				line = actorIn . getLine ();
			}
		}


		else if ( line == SCRIPT_STATS ) {
			while ( true ) {
				line = actorIn . getLine ();
				if ( line [ 0 ] == SCRIPT_HEADER )
					break;
				string statName = line . substr ( 0, line . find ( ' ' ) );
				int statVal = atoi ( line . substr ( line . find ( ' ' ) + 1, string::npos ) . c_str () );
				stats [ statName ] = statVal;
			}
		}

		else if ( line == SCRIPT_IMG ) {
			while (	true ) {
				line = actorIn . getLine ();

				if ( line == SCRIPT_IMG_FRONT ) {
					line = actorIn . getLine ();
					if ( ! mapObj )
						mapObj = new FaceObj ( actorDir + line, x, y, s );
					else
						mapObj -> loadFace ( s, actorDir + line );
				} else if ( line == SCRIPT_IMG_BACK ) {
					line = actorIn . getLine ();
					if ( ! mapObj )
						mapObj = new FaceObj ( actorDir + line, x, y, n );
					else
						mapObj -> loadFace ( n, actorDir + line );
				} else if ( line == SCRIPT_IMG_RIGHT ) {
					line = actorIn . getLine ();
					if ( ! mapObj )
						mapObj = new FaceObj ( actorDir + line, x, y, e );
					else
						mapObj -> loadFace ( e, actorDir + line );
				} else if ( line == SCRIPT_IMG_LEFT ) {
					line = actorIn . getLine ();
					if ( ! mapObj )
						mapObj = new FaceObj ( actorDir + line, x, y, w );
					else
						mapObj -> loadFace ( w, actorDir + line );
				} else if ( line == SCRIPT_IMG_NIL ) {
					line = actorIn . getLine ();
					if ( ! mapObj )
						mapObj = new FaceObj ( actorDir + line, x, y, nil );
					else
						mapObj -> loadFace ( nil, actorDir + line );
				} else
					break;
			}
		}

		else if ( line == SCRIPT_SCRIPTS ) {
			while ( ! actorIn . eof () ) {
				scripts . addScript ( actorIn . getLine () );
			}
		}

		else
			fatErrMsg ( "Actor script file from " + actorDir + SCRIPTFILE + " is improperly formatted on line " + line + "." );
	}

	dead = false;
}

void Actor::run () {
	if ( ! dead )
		scripts . run ();
}

void Actor::die () {
	dead = true;
}