#include "actor.h"
#include "script.h"
#include "event.h"
#include "map.h"
#include "obj.h"
#include "gen.h"
#include <string>
#include <vector>
#include <cstdlib>
using namespace std;

const string COND_FLAG ( "flag" );
const string COND_NOTFLAG ( "notFlag" );
const string COND_TALKEDTO ( "talkedTo" );
const string COND_STATLESS ( "statLess" );
const string COND_STATMORE ( "statMore" );



// Flag

Flag::Flag ( Actor * self, string flagName )
: Cond ( self ) {
	flagId = eventList . retIndex ( flagName );
}

bool Flag::eval () {
	return eventList [ flagId ];
}



// NotFlag

NotFlag::NotFlag ( Actor * self, string flagName )
: Cond ( self ) {
	flagId = eventList . retIndex ( flagName );
}

bool NotFlag::eval () {
	return ( ! eventList [ flagId ] );
}



// TalkedTo

TalkedTo::TalkedTo ( Actor * self )
: Cond ( self ) {
	;
}

bool TalkedTo::eval () {
	if ( self -> retTalkedTo () ) {
		self -> setTalkedTo ( false );
		return true;
	}
	return false;
}



// StatLess

StatLess::StatLess ( Actor * self, string args )
: Cond ( self ) {
	if ( args . find ( ARGDIV ) == string::npos )
		fatErrMsg ( "Not enough arguments in \"" + COND_STATLESS + "\" condition with arguments \"" + args + "\"." );

	statName = args . substr ( 0, args . find ( ARGDIV ) );
	statVal = atoi ( args . substr ( args . find ( ARGDIV ), string::npos ) . c_str () );
}

bool StatLess::eval () {
	if ( self -> stats [ statName ] < statVal )
		return true;
	return false;
}



// StatMore

StatMore::StatMore ( Actor * self, string args )
: Cond ( self ) {
	if ( args . find ( ARGDIV ) == string::npos )
		fatErrMsg ( "Not enough arguments in \"" + COND_STATMORE + "\" condition with arguments \"" + args + "\"." );

	statName = args . substr ( 0, args . find ( ARGDIV ) );
	statVal = atoi ( args. substr ( args . find ( ARGDIV ), string::npos ) . c_str () );
}

bool StatMore::eval () {
	if ( self -> stats [ statName ] > statVal )
		return true;
	return false;
}



// CondList

string CondList::stripArg ( string & line ) {
	const Strind argStart = line . find ( ARG1 ) + 1;
	const Strind argEnd = line . find ( ARG2 ) - 1;
	if ( argStart > argEnd ) {
		line = line . substr ( argEnd + 2, string::npos );
		string emptyArg;
		return emptyArg;
	}
	const Strind argLength = argEnd - argStart + 1;
	string lineArg = line . substr ( argStart, argLength );
	line = line . substr ( argEnd + 2, string::npos );
	return lineArg;
}

CondList::CondList ( Actor * self, string line ) {
	if ( line . find ( ARG1 ) == string::npos || line . find ( ARG2 ) == string::npos )
		fatErrMsg ( "Actor script improperly formatted at line " + line + ": no argument." );

	while ( line . find ( ARG1 ) != string::npos ) {
		string condType = line . substr ( 0, line . find ( ARG1 ) );
		if ( condType == COND_FLAG ) {
			string flagName = stripArg ( line );
			Flag * flag = new Flag ( self, flagName );
			conds . push_back ( flag );
		} else if ( condType == COND_NOTFLAG ) {
			string flagName = stripArg ( line );
			NotFlag * notFlag = new NotFlag ( self, flagName );
			conds . push_back ( notFlag );
		} else if ( condType == COND_TALKEDTO ) {
			stripArg ( line );
			TalkedTo * talkedTo = new TalkedTo ( self );
			conds . push_back ( talkedTo );
		} else if ( condType == COND_STATLESS ) {
			string args = stripArg ( line );
			StatLess * statLess = new StatLess ( self, args );
			conds . push_back ( statLess );
		} else if ( condType == COND_STATMORE ) {
			string args = stripArg ( line );
			StatMore * statMore = new StatMore ( self, args );
			conds . push_back ( statMore );
		} else
			fatErrMsg ( "Actor script improperly formatted at line segment \"" + line + "\": non-existent condition used." );
	}
}

CondList::~CondList () {
	for ( Conds::iterator it = conds . begin (); it != conds . end (); it++ )
		delete (*it);
}

bool CondList::eval () {
	for ( Conds::iterator it = conds . begin (); it != conds . end (); it++ )
		if ( ! ( (*it) -> eval () ) )
			return false;
	return true;
}