#ifndef HEAD_GUI
#define HEAD_GUI

#include "lib.h"
#include "gen.h"
#include <string>
#include <vector>

extern Lib_Clr curTextClr;
extern Lib_Clr curSelTextClr;

const int RIGHTALIGN = -1;
const int BOTTOMALIGN = -1;
const int CENTER = -2;

void initGui ();

class GuiBox {

	private:

	int x1, y1, x2, y2;
	Bmp boxBmp;

	void loadBmp ();

	public:

	GuiBox ( int x1, int y1, int x2, int y2 );
	~GuiBox () { ; }

	inline int retX1 () const { return x1; }
	inline int retY1 () const { return y1; }
	inline int retX2 () const { return x2; }
	inline int retY2 () const { return y2; }

	inline void draw () const { boxBmp . draw ( x1, y1 ); }
};

class TextBox {

	protected:

	std::vector <std::string> text;

	GuiBox * box;
	Font * font;
	Lib_Clr textClr;

	public:

	TextBox ( std::string text, int x, int y, Font * font = curFont, Lib_Clr textClr = curTextClr );
	~TextBox () { delete box; }

	inline int retX1 () const { return box -> retX1 (); }
	inline int retY1 () const { return box -> retY1 (); }
	inline int retX2 () const { return box -> retX2 (); }
	inline int retY2 () const { return box -> retY2 (); }

	virtual void draw () const;
};

class MsgBox : public TextBox {

	public:

	MsgBox ( std::string text, int x, int y, Font * font = curFont, Lib_Clr textClr = curTextClr );
	~MsgBox () { ; }
};

class Btn : public TextBox {

	protected:

	bool sel;
	Lib_Clr selTextClr;

	public:

	Btn ( std::string text, int x, int y, Font * font = curFont, Lib_Clr textClr = curTextClr, Lib_Clr selTextClr = curSelTextClr );
	~Btn () { ; }

	inline void setSel ( bool sel ) { Btn::sel = sel; }
	inline bool retSel () const { return sel; }

	void draw () const;
};



// EmptyBox

class EmptyBox {

	protected:

	GuiBox * box;

	public:

	EmptyBox ( int x1, int y1, int x2, int y2 );
	~EmptyBox () { delete box; }

	inline int retX1 () { return box -> retX1 (); }
	inline int retY1 () { return box -> retY1 (); }
	inline int retX2 () { return box -> retX2 (); }
	inline int retY2 () { return box -> retY2 (); }

	virtual void draw ();
};

class Menu : public EmptyBox {

	protected:

	int curSel;

	typedef std::vector <Btn *> Btns;
	Btns btns;

	public:

	Menu ( std::string text, int x1, int y1, int x2, int y2, Font * font = curFont, Lib_Clr textClr = curTextClr, Lib_Clr selTextClr = curSelTextClr );
	~Menu ();

	void draw ();
	int run ();
};



// VoxBox

class VoxBox {

	protected:

	std::vector <std::string> text;

	GuiBox * box;
	Font * font;
	Lib_Clr textClr;

	public:

	VoxBox ( std::string text, Font * font = curFont, Lib_Clr textClr = curTextClr );
	~VoxBox () { delete box; }

	inline int retX1 () const { return box -> retX1 (); }
	inline int retY1 () const { return box -> retY1 (); }
	inline int retX2 () const { return box -> retX2 (); }
	inline int retY2 () const { return box -> retY2 (); }

	virtual int run ();
};

class ChoiceBox : public VoxBox {

	protected:

	int curSel;
	std::vector <Btn *> btns;

	public:

	ChoiceBox ( std::string text, Font * font = curFont, Lib_Clr textClr = curTextClr, Lib_Clr selTextClr = curSelTextClr );
	~ChoiceBox () { for ( unsigned int i = 0; i < btns . size (); i++ ) delete btns [ i ]; }

	int run ();
};

#endif