#include "img.h"
#include "lib.h"
#include "gen.h"
#include <map>
#include <string>
using namespace std;

ImgList * imgList = NULL;

ImgList::ImgList () {
	if ( ! libLoaded )
		fatErrMsg ( "ImgList constructor called before library loaded." );

	loaded = false;
}

ImgList::~ImgList () {
	if ( loaded )
		fatErrMsg ( "Destructor called on still-loaded ImgList." );
}

void ImgList::load ( string fileName ) {
	if ( loaded )
		fatErrMsg ( "Attempt to reload ImgList from " + fileName + "." );

	FileIn listIn ( fileName . c_str () );

	while ( ! listIn . eof () ) {
		string line = listIn . getLine ();
		string idString = line . substr ( 0, line . find ( ' ' ) );
		string fileString = line . substr ( line . find ( ' ' ) + 1, string::npos );

		Sprite * sprite = new Sprite ( fileString );
		spriteMap [ atoi ( idString . c_str () ) ] = sprite;
	}

	loaded = true;
}

void ImgList::destroy () {
	if ( ! loaded )
		fatErrMsg ( "Attempt to destroy unloaded ImgList." );

	for ( SpriteMap::iterator it = spriteMap . begin (); it != spriteMap . end (); it++ )
		delete (*it) . second;
	spriteMap . clear ();

	loaded = false;
}

int ImgList::addImg ( Sprite * sprite ) {
	if ( ! loaded )
		fatErrMsg ( "Attempt to add individual img before ImgList is loaded." );

	for ( int i = 0; i < INT_MAX; i++ ) {
		if ( spriteMap . find ( i ) == spriteMap . end () ) {
			spriteMap [ i ] = sprite;
			return i;
		}
	}
	fatErrMsg ( "Unable to create new element for ImgList." );
	return 0;
}

Sprite * ImgList::retImg ( int imgId ) {
	if ( ! loaded )
		fatErrMsg ( "Attempt to index ImgList when it is unloaded." );

	if ( spriteMap . find ( imgId ) == spriteMap . end () )
		fatErrMsg ( "Attempt to index non-existent element " + numToString ( imgId ) + " in spriteMap." );
	return ( * spriteMap . find ( imgId ) ) . second;
}