#include "lib.h"

#ifdef BCC32
	#include "bccalleg.h"
#endif
#ifdef MING
	#include <allegro.h>
#endif

#ifdef FULLSCREEN
	#define GFXCARD GFX_AUTODETECT_FULLSCREEN
#endif
#ifdef WINDOWED
	#define GFXCARD GFX_AUTODETECT_WINDOWED
#endif
#define GOODGFXMODE 0 // A successful call of the Allegro function "set_gfx_mode" will return this value

#include "fonts.h"
#include "keydefs.h"
#include "gui.h"
#include "gen.h"
#include <string>
#include <math.h>
#include <stdlib.h>
using namespace std;

int screenW = INISCREENW;
int screenH = INISCREENH;

void fatErrMsg ( string msg ) {
	if ( libLoaded ) {
		set_gfx_mode ( GFX_TEXT, 0, 0, 0, 0 );

		msg = PROGRAMNAME "\n\nFatal error: " + msg;
		allegro_message ( msg . c_str () );

		exit ( EXIT_FAILURE );
	} else {
		allegro_init ();

		msg = PROGRAMNAME "\nFatal error: " + msg;
		allegro_message ( msg . c_str () );

		exit ( EXIT_FAILURE );
	}
}

void errMsg ( string msg ) {
	MsgBox msgBox ( msg, 50, 50 );
}



// Lib

bool libLoaded = false;

void initLib () {
	allegro_init ();
	install_timer ();

	libLoaded = true;

	#ifdef WINDOWED
		set_window_title ( PROGRAMNAME );
	#endif

	setScreenRes ( INISCREENW, INISCREENH );

	#ifdef FULLSCREEN
		if ( set_display_switch_mode ( SWITCHMODE ) < 0 )
			fatErrMsg ( "Could not set switch mode." );
		void ( * drawScreenPointer ) () = drawScreen;
		set_display_switch_callback ( SWITCH_IN, drawScreenPointer );
	#endif

	keyboard = new Keyboard ();
	#ifndef NOMOUSE
		mouse = new Mouse ();
	#endif

	text_mode ( -1 );

	Datafile fontDat ( "data\\fonts.dat" );

	font1 = new Font ( fontDat . extractFont ( 0 ) );
	curFont = INICURFONT;
}



// Keyboard

Keyboard * keyboard = NULL;

Keyboard::Keyboard () {
	if ( ! libLoaded )
		fatErrMsg ( "Attempt to install keyboard before library has been installed." );

	install_keyboard ();
}

bool Keyboard::press ( Lib_Key keyID ) const {
	if ( key [ keyID ] == TRUE )
		return true;
	return false;
}

bool Keyboard::tap ( Lib_Key keyID ) const {

	// Needs work

	if ( press ( keyID ) ) {
		while ( press ( keyID ) ) { ; }
		return true;
	}
	return false;
}

void Keyboard::waitForTap ( Lib_Key keyID ) const {
	while ( press ( keyID ) ) { ; }
	while ( ! press ( keyID ) ) { ; }
	while ( press ( keyID ) ) { ; }
}

Compass Keyboard::dir () const {
	const Lib_Key u = key [ KEY_UP ];
	const Lib_Key r = key [ KEY_RIGHT ];
	const Lib_Key d = key [ KEY_DOWN ];
	const Lib_Key l = key [ KEY_LEFT ];

	if ( u && !r && !d && !l )
		return n;
	else if ( u && r && !d && !l )
		return ne;
	else if ( r && !u && !d && !l )
		return e;
	else if ( d && r && !u && !l )
		return se;
	else if ( d && !u && !r && !l )
		return s;
	else if ( d && l && !u && !r )
		return sw;
	else if ( l && !u && !d && !r )
		return w;
	else if ( u && l && ! d && ! r )
		return nw;
	else
		return nil;
}

string Keyboard::textIn ( int numChars ) const {
	string temp = "Keyboard::textIn not completed yet.";
	return temp;
}

int Keyboard::numIn ( int numDigits ) const {
	return 0;
}



// Mouse
#ifndef NOMOUSE
	Mouse * mouse = NULL;

	Mouse::Mouse () {
		if ( ! libLoaded )
			fatErrMsg ( "Attempt to install mouse before library has been install." );

		int numBtns = install_mouse ();

		if ( numBtns < MINMOUSEBTNS ) {
			fatErrMsg ( "Sorry; this game requires that you have at least " + numToString ( MINMOUSEBTNS ) + " buttons." );
		}

		if ( mouse_needs_poll () == TRUE ) {
			needPoll = true;
		} else {
			needPoll = false;
		}

		show_mouse ( screen );
	}

	bool Mouse::click () const {
		for ( int i = 1; i <= MINMOUSEBUTTONS; i ++ ) {
			if ( button ( i ) ) {
				return true;
			}
		}
		return false;
	}

	bool Mouse::inField ( int x1, int y1, int x2, int y2 ) const {
		const int X = x ();
		const int Y = y ();

		if ( x1 <= X && y1 <= Y && X <= x2 && Y <= y2 ) {
			return true;
		}
		return false;
	}
#endif



// Graphics



// Screen

Bmp * screenBuff;

void drawScreen () {
	screenBuff -> drawToScreen ( 0, 0 );
}

void clearScreenToClr ( Lib_Clr colour ) {
	screenBuff -> clearToClr ( colour );
}

Lib_Clr clrAtScreenPxl ( int x, int y ) {
	return screenBuff -> clrAtPxl ( x, y );
}

void setScreenRes ( int newScreenW, int newScreenH ) {
	bool goodRes = false;

	set_color_depth ( 32 );
	if ( set_gfx_mode ( GFXCARD, newScreenW, newScreenH, 0, 0 ) == GOODGFXMODE )
		goodRes = true;

	if ( ! goodRes ) {
		set_color_depth ( 16 );
		if ( set_gfx_mode ( GFXCARD, newScreenW, newScreenH, 0, 0 ) == GOODGFXMODE )
			goodRes = true;
	}
/*
	if ( ! goodRes ) {
		set_color_depth ( 15 );
		if ( set_gfx_mode ( GFXCARD, newScreenW, newScreenH, 0, 0 ) == GOODGFXMODE )
			goodRes = true;
	}
*/
	if ( ! goodRes ) {
		set_color_depth ( 24 );
		if ( set_gfx_mode ( GFXCARD, newScreenW, newScreenH, 0, 0 ) == GOODGFXMODE )
			goodRes = true;
	}

	if ( ! goodRes ) {
		set_color_depth ( 8 );
		if ( set_gfx_mode ( GFXCARD, newScreenW, newScreenH, 0, 0 ) == GOODGFXMODE )
			goodRes = true;
	}

	if ( goodRes ) {
		screenW = newScreenW;
		screenH = newScreenH;
		if ( screenBuff == NULL )
			screenBuff = new Bmp ( screenW, screenH );
		else
			screenBuff -> reload ( screenW, screenH );
	} else {
		if ( set_gfx_mode ( GFXCARD, screenW, screenH, 0, 0 ) == GOODGFXMODE )
			errMsg ( "Could not set graphics mode " + numToString ( newScreenW ) + "x" + numToString ( newScreenH ) );
		else
			fatErrMsg ( "Could not reset graphics mode to previous settings of " + numToString ( screenW ) + "x" + numToString ( screenH ) );
	}
}



// Bmp

Bmp::Bmp ( int width, int height ) {
	if ( ! libLoaded )
		fatErrMsg ( "Attempt to load " + numToString ( width ) + "x" + numToString ( height ) + " bitmap before library has been installed." );

	theBmp = create_bitmap ( width, height );
	if ( theBmp == NULL )
		fatErrMsg ( "Error creating new " + numToString ( width ) + "x" + numToString ( height ) + " bitmap." );
	clearToClr ( DEFBMPCLR );
}

Bmp::Bmp ( string bmpFile ) {
	if ( ! libLoaded )
		fatErrMsg ( "Attempt to load bitmap from " + bmpFile + " before library has been installed." );

	theBmp = load_bmp ( bmpFile . c_str (), thePal );
	if ( theBmp == NULL )
		fatErrMsg ( "Error loading bitmap from " + bmpFile );
}

Bmp::Bmp ( Lib_Bmp libBmp ) {
	if ( ! libLoaded )
		fatErrMsg ( "Attempt to load bitmap from library id before library has been installed." );

	theBmp = libBmp;
	if ( ! ( is_memory_bitmap ( theBmp ) == TRUE ) )
		fatErrMsg ( "Error loading bitmap from library id." );
}

Bmp::~Bmp () {
	destroy_bitmap ( theBmp );
}

void Bmp::reload ( int width, int height ) {
	destroy_bitmap ( theBmp );
	theBmp = create_bitmap ( width, height );
	if ( theBmp == NULL )
		fatErrMsg ( "Error creating new " + numToString ( width ) + "x" + numToString ( height ) + " bitmap." );
	clearToClr ( DEFBMPCLR );
}

void Bmp::reload ( string bmpFile ) {
	destroy_bitmap ( theBmp );
	theBmp = load_bmp ( bmpFile . c_str (), thePal );
	if ( theBmp == NULL )
		fatErrMsg ( "Error loading bitmap from " + bmpFile );
}

void Bmp::drawToScreen ( int x, int y ) const {
	acquire_screen ();
	blit ( theBmp, screen, 0, 0, x, y, retW (), retH () );
	release_screen ();
}

void Bmp::drawToBmp ( Bmp & targBmp, int x, int y ) const {
	blit ( theBmp, targBmp . theBmp, 0, 0, x, y, retW (), retH () );
}

void Bmp::drawWarpedToBmp ( Bmp & targBmp, int targX1, int targY1, int targX2, int targY2 ) const {
	const int TARGW = targX2 - targX1 + 1;
	const int TARGH = targY2 - targY1 + 1;
	stretch_blit ( theBmp, targBmp . theBmp, 0, 0, retW (), retH (), targX1, targY1, TARGW, TARGH );
}

void Bmp::drawPartToBmp ( Bmp & targBmp, int targX, int targY, int srcX1, int srcY1, int srcX2, int srcY2 ) const {
	const int PARTW = srcX2 - srcX1 + 1;
	const int PARTH = srcY2 - srcY1 + 1;
	blit ( theBmp, targBmp . theBmp, srcX1, srcY1, targX, targY, PARTW, PARTH );
}

void Bmp::drawMaskToBmp ( Bmp & targBmp, int x, int y ) const {
	masked_blit ( theBmp, targBmp . theBmp, 0, 0, x, y, retW (), retH () );
}

void Bmp::drawMaskWarpedToBmp ( Bmp & targBmp, int targX1, int targY1, int targX2, int targY2 ) const {
	const int TARGW = targX2 - targX1 + 1;
	const int TARGH = targY2 - targY1 + 1;
	masked_stretch_blit ( theBmp, targBmp . theBmp, 0, 0, retW (), retH (), targX1, targY1, TARGW, TARGH );
}

void Bmp::drawMaskPartToBmp ( Bmp & targBmp, int targX, int targY, int srcX1, int srcY1, int srcX2, int srcY2 ) const {
	const int PARTW = srcX2 - srcX1 + 1;
	const int PARTH = srcY2 - srcY1 + 1;
	masked_blit ( theBmp, targBmp . theBmp, srcX1, srcY1, targX, targY, PARTW, PARTH );
}


void Bmp::tileToBmp ( Bmp & targBmp, int targX1, int targY1, int targX2, int targY2 ) const {
	const int TARGW = targX2 - targX1 + 1;
	const int TARGH = targY2 - targY1 + 1;
	const int NUMROWS = int ( TARGH / retH () );
	const int NUMCOLS = int ( TARGW / retW () );

	for ( int row = 0; row < NUMROWS; row++ ) {
		for ( int col = 0; col < NUMCOLS; col++ ) {
			const int TARGX = targX1 + ( col * retW () );
			const int TARGY = targY1 + ( row * retH () );
			drawToBmp ( targBmp, TARGX, TARGY );
		}
		const int TARGX = targX1 + ( NUMCOLS * retW () );
		const int TARGY = targY1 + ( row * retH () );
		const int SRCX1 = 0;
		const int SRCY1 = 0;
		const int SRCX2 = TARGW - ( NUMCOLS * retW () ) - 1;
		const int SRCY2 = retH () - 1;
		drawPartToBmp ( targBmp, TARGX, TARGY, SRCX1, SRCY1, SRCX2, SRCY2 );
	}
	for ( int col = 0; col < NUMCOLS; col++ ) {
		const int TARGX = targX1 + ( col * retW () );
		const int TARGY = targY1 + ( NUMROWS * retH () );
		const int SRCX1 = 0;
		const int SRCY1 = 0;
		const int SRCX2 = retW () - 1;
		const int SRCY2 = TARGH - ( NUMROWS * retH () ) - 1;
		drawPartToBmp ( targBmp, TARGX, TARGY, SRCX1, SRCY1, SRCX2, SRCY2 );
	}
	const int TARGX = targX1 + ( NUMCOLS * retW () );
	const int TARGY = targY1 + ( NUMROWS * retH () );
	const int SRCX1 = 0;
	const int SRCY1 = 0;
	const int SRCX2 = TARGW - ( NUMCOLS * retW () ) - 1;
	const int SRCY2 = TARGH - ( NUMROWS * retH () ) - 1;
	drawPartToBmp ( targBmp, TARGX, TARGY, SRCX1, SRCY1, SRCX2, SRCY2 );
}

void Bmp::tileMaskToBmp ( Bmp & targBmp, int targX1, int targY1, int targX2, int targY2 ) const {
	const int TARGW = targX2 - targX1 + 1;
	const int TARGH = targY2 - targY1 + 1;

	int horzCntr;
	for ( horzCntr = 0; ( ( horzCntr + 1 ) * retW () ) < TARGW; horzCntr++ ) {
		int vertCntr;
		for ( vertCntr = 0; ( ( vertCntr + 1 ) * retH () ) < TARGH; vertCntr++ ) {
			const int BMPX = targX1 + horzCntr * retW () ;
			const int BMPY = targY1 + vertCntr * retH () ;
			drawMaskToBmp ( targBmp, BMPX, BMPY );
		}
		const int TARGX = targX1 + horzCntr * retW ();
		const int TARGY = targY1 + horzCntr * retH ();
		const int SRCX1 = 0;
		const int SRCY1 = 0;
		const int SRCX2 = SRCX1 + retW () - 1;
		const int SRCY2 = TARGH - ( vertCntr * retH () );
		drawMaskPartToBmp ( targBmp, TARGX, TARGY, SRCX1, SRCY1, SRCX2, SRCY2 );
	}
	int vertCntr;
	for ( vertCntr = 0; ( ( vertCntr + 1 ) * retH () ) < TARGH; vertCntr++ ) {
		const int TARGX = targX1 + horzCntr * retW ();
		const int TARGY = targY1 + horzCntr * retH ();
		const int SRCX1 = 0;
		const int SRCY1 = 0;
		const int SRCX2 = TARGW - ( horzCntr * retW () );
		const int SRCY2 = SRCY1 + retH () - 1;
		drawMaskPartToBmp ( targBmp, TARGX, TARGY, SRCX1, SRCY1, SRCX2, SRCY2 );
	}
	const int TARGX = targX1 + horzCntr * retW ();
	const int TARGY = targY1 + horzCntr * retH ();
	const int SRCX1 = 0;
	const int SRCY1 = 0;
	const int SRCX2 = TARGW - ( horzCntr * retW () );
	const int SRCY2 = TARGH - ( vertCntr * retH () );
	drawPartToBmp ( targBmp, TARGX, TARGY, SRCX1, SRCY1, SRCX2, SRCY2 );
}



// Sprite

Sprite::Sprite ( Lib_Sprite libID, int argSpriteW, int argSpriteH ) {
	if ( ! libLoaded )
		fatErrMsg ( "Attempt to load " + numToString ( argSpriteW ) + "x" + numToString ( argSpriteH ) + " Sprite from library id before library has been installed." );
	/*
		Warning:
			I don't know how to do any error checking at all on these
			sprites, which is really sad, but hey, what are you going
			to do, eh?
	*/
	spriteW = argSpriteW;
	spriteH = argSpriteH;
	theSprite = libID;
	fatErrMsg ( "The method of loading sprites directly from a libID is not currently supported." );
}

Sprite::Sprite ( string fileName ) {
	if ( ! libLoaded )
		fatErrMsg ( "Attempt to load Sprite from library id before library has been installed." );

	Bmp srcBmp ( fileName );
	spriteW = srcBmp . retW ();
	spriteH = srcBmp . retH ();
	theSprite = get_rle_sprite ( srcBmp . theBmp );
}

Sprite::~Sprite () {
	destroy_rle_sprite ( theSprite );
}

void Sprite::drawToBmp ( Bmp & targBmp, int x, int y ) const {
	draw_rle_sprite ( targBmp . theBmp, theSprite, x, y );
}



// Fonts

Font * font1;
Font * curFont;

Font::Font ( Lib_Font fontID ) {
	if ( ! libLoaded )
		fatErrMsg ( "Attempt to load font from library id before library has been installed." );
	theFont = fontID;
}

Font::~Font () {
	destroy_font ( theFont );
}

string Font::wrapToWidth ( string text, int width ) {
	string retText;

	while ( true ) {
		if ( retW ( text ) <= width ) {
			retText += text;
			break;
		} else {
			for ( Strind i = text . size (); i > 1; i-- ) {
				if ( retW ( text . substr ( 0, i ) ) < width && text [ i - 1 ] == ' ' ) {
					retText += text . substr ( 0, i ) + '\n';
					text = text . substr ( i, string::npos );
					break;
				}
			}
		}
	}

	return retText;
}

void Font::drawToBmp ( Bmp & targBmp, string text, int x, int y, Lib_Clr clr ) const {
	textout ( targBmp . theBmp, theFont, text . c_str (), x, y, clr );
}

// Datafile

Datafile::Datafile ( string datafile ) {
	if ( ! libLoaded )
		fatErrMsg ( "Attempt to load datafile from " + datafile + " before library has been installed." );

	theDatafile = load_datafile ( datafile . c_str () );
	if ( theDatafile == NULL ) {
		fatErrMsg ( "Could not open datafile from " + datafile + "." );
	}
}

Datafile::~Datafile () {
	unload_datafile ( theDatafile );
}

Lib_Bmp Datafile::extractBmp ( int dataIndex ) const {
	return ( Lib_Bmp ) theDatafile [ dataIndex ] . dat;
}

Lib_Sprite Datafile::extractSprite ( int dataIndex ) const {
	return ( Lib_Sprite ) theDatafile [ dataIndex ] . dat;
}

Lib_Font Datafile::extractFont ( int dataIndex ) const {
	if ( theDatafile [ dataIndex ] . dat == NULL )
		fatErrMsg ( "Error loading font from datafile." );
	return ( Lib_Font ) theDatafile [ dataIndex ] . dat;
}