#ifndef HEAD_LIB
#define HEAD_LIB

#ifdef BCC
	#include "bccalleg.h"
#endif
#ifdef MING
	#include <allegro.h>
#endif

#include "keydefs.h"
#include "gen.h"
#include <string>

//#define FULLSCREEN
#define WINDOWED
#define NOMOUSE

#ifdef FULLSCREEN
	#ifdef WINDOWED
		blllaaargh!
	#endif
#endif

#define STRIPPED

#define PROGRAMNAME "Dahl"

#define DATADIR "data/"
#define FONTDATAFILE DATADIR "fonts.dat"

#ifdef FULLSCREEN
	const int SWITCHMODE = SWITCH_AMNESIA;
#endif

void fatErrMsg ( std::string errorMessage );
void errMsg ( std::string errorMessage );



// Lib

extern bool libLoaded;

void initLib ();



// Keyboard
		
typedef int Lib_Key;

/*
	For key IDs, look in "keydefs.h".  Important ones include:
		Escape key:  KEY_ESC
		Enter key:   KEY_ENTER
		Space key:	 KEY_SPACE
*/

class Keyboard {

	public:

	Keyboard ();

	inline bool anyPressed () const { return keypressed (); }
	bool press ( Lib_Key keyID ) const;
	bool tap ( Lib_Key keyID ) const;

	void waitForTap ( Lib_Key keyID ) const;
	void waitForRelease ( Lib_Key keyId ) const;
	Compass dir () const;

	std::string textIn ( int numChars ) const;
	int numIn ( int numDigits ) const;
};
extern Keyboard * keyboard;

class Bmp;
class Sprite;
class Font;

typedef BITMAP * Lib_Bmp;
typedef FONT * Lib_Font;
typedef RLE_SPRITE * Lib_Sprite;

typedef DATAFILE * Lib_Datafile;

class Datafile {

	private:

	Lib_Datafile theDatafile;

	public:

	Datafile ( std::string fileName );
	~Datafile ();

	Lib_Bmp extractBmp ( int dataIndex ) const;
	Lib_Sprite extractSprite ( int dataIndex ) const;
	Lib_Font extractFont ( int dataIndex ) const;
};




// Graphics

typedef int Lib_Clr;

const int INISCREENW = 800;
const int INISCREENH = 600;
const int INICLRDEPTH = 8;

extern int screenW;
extern int screenH;

void setScreenRes ( int screenW, int screenH );

inline Lib_Clr createClr ( int red, int green, int blue ) {
	makecol ( red, green, blue );
}

class Bmp;
extern Bmp * screenBuff;

void drawScreen ();
void clearScreenToClr ( Lib_Clr clr );
Lib_Clr clrAtScreenPxl ( int x, int y );



// Font

typedef FONT * Lib_Font;

const Lib_Clr DEFFONTCLR = createClr ( 255, 255, 255 );

class Font;
extern Font * font1;
extern Font * curFont;

#define INICURFONT font1

class Font {


	friend Lib_Font Datafile::extractFont ( int ) const;
	private:

	Lib_Font theFont;

	public:

	Font ( Lib_Font fontID );
	~Font ();

	inline int retW ( std::string text ) const { return text_length ( theFont, text . c_str () ); }
	inline int retH () const { return text_height ( theFont ); }

	std::string wrapToWidth ( std::string text, int width );

	void drawToBmp ( Bmp & targetBmp, std::string text, int x, int y, Lib_Clr clr = DEFFONTCLR ) const;
	inline void draw ( std::string text, int x, int y, Lib_Clr clr = DEFFONTCLR ) const { drawToBmp ( * screenBuff, text, x, y, clr ); }
};



// Bmp


typedef BITMAP * Lib_Bmp;
typedef PALETTE Lib_Pal;

const Lib_Clr DEFBMPCLR = createClr ( 0, 0, 0 );

class Bmp;
extern Bmp * screenBuff;
class Sprite;
class Bmp {


	friend Lib_Bmp Datafile::extractBmp ( int ) const;
	friend void Font::drawToBmp ( Bmp &, std::string, int, int, Lib_Clr ) const;
	friend class Sprite;
	private:

	Lib_Bmp theBmp;
	#if CLRDEPTH == 8
		Lib_Pal thePal;
	#else
		#define thePal NULL
	#endif

	public:

	Bmp ( const Bmp & src ) {
		const int BMPW = src . retW ();
		const int BMPH = src . retH ();
		create_bitmap ( BMPW, BMPH );
		blit ( src . theBmp, theBmp, 0, 0, 0, 0, BMPW, BMPH );
	}

	Bmp ( int width, int height );
	Bmp ( std::string fileName );
	Bmp ( Lib_Bmp libBmp );

	~Bmp ();

	void reload ( int width, int height );
	void reload ( std::string fileName );

	inline int retW () const { return theBmp -> w; }
	inline int retH () const { return theBmp -> h; }

	inline void clearToClr ( Lib_Clr clr ) { clear_to_color ( theBmp, clr ); }
	inline Lib_Clr clrAtPxl ( int x, int y ) const { return getpixel ( theBmp, x, y ); }

	void drawToScreen ( int x, int y ) const;

	void drawToBmp ( Bmp & targBmp, int x, int y ) const;
	void drawWarpedToBmp ( Bmp & targBmp, int targX1, int targY1, int targX2, int targY2 ) const;
	void drawPartToBmp ( Bmp & targBmp, int targX, int targY, int srcX1, int srcY1, int srcX2, int srcY2 ) const;

	void drawMaskToBmp ( Bmp & targBmp, int x, int y ) const;
	void drawMaskWarpedToBmp ( Bmp & targBmp, int targX1, int targY1, int targX2, int targY2 ) const;
	void drawMaskPartToBmp ( Bmp & targBmp, int targX, int targY, int srcX1, int srcY1, int srcX2, int srcY2 ) const;

	void tileToBmp ( Bmp & targBmp, int targX1, int targY1, int targX2, int targY2 ) const;
	void tileMaskToBmp ( Bmp & targBmp, int targX1, int targY1, int targX2, int targY2 ) const;

	inline void draw ( int x, int y ) const { drawToBmp ( * screenBuff, x, y ); }
	inline void drawWarped ( int targX1, int targY1, int targX2, int targY2 ) const { drawWarpedToBmp ( * screenBuff, targX1, targY1, targX2, targY2 ); }
	inline void drawPart ( int targX, int targY, int srcX1, int srcY1, int srcX2, int srcY2 ) const { drawPartToBmp ( * screenBuff, targX, targY, srcX1, srcY1, srcX2, srcY2 ); }

	inline void drawMask ( int x, int y ) const { drawMaskToBmp ( * screenBuff, x, y ); }
	inline void drawMaskWarped ( int targX1, int targY1, int targX2, int targY2 ) const { drawMaskWarpedToBmp ( * screenBuff, targX1, targY1, targX2, targY2 ); }
	inline void drawMaskPart ( int targX, int targY, int srcX1, int srcY1, int srcX2, int srcY2 ) const { drawMaskPartToBmp ( * screenBuff, targX, targY, srcX1, srcY1, srcX2, srcY2 ); }

	inline void tile ( int targX1, int targY1, int targX2, int targY2 ) const { tileToBmp ( * screenBuff, targX1, targY1, targX2, targY2 ); }
	inline void tileMask ( int targX1, int targY1, int targX2, int targY2 ) const { tileMaskToBmp ( * screenBuff, targX1, targY1, targX2, targY2 ); }
};



// Sprite

typedef RLE_SPRITE * Lib_Sprite;

class Sprite {

	friend Lib_Sprite Datafile::extractSprite ( int ) const;
	private:

	int spriteW, spriteH;
	Lib_Sprite theSprite;

	public:

	Sprite ( const Sprite & src ) {
		spriteW = src . retW ();
		spriteH = src . retH ();

		Bmp tempBmp ( spriteW, spriteH );
		src . drawToBmp ( tempBmp, 0, 0 );
		theSprite = get_rle_sprite ( tempBmp . theBmp );
	}

	inline int retW () const { return spriteW; }
	inline int retH () const { return spriteH; }

	Sprite ( std::string fileName );
	Sprite ( Lib_Sprite libSprite, int width, int height );

	~Sprite ();

	void drawToBmp ( Bmp & targBmp, int x, int y ) const;
	inline void draw ( int x, int y ) const { drawToBmp ( * screenBuff, x, y ); }
};

#endif