#include "map.h"
#include "obj.h"
#include "lib.h"
#include "gen.h"
#include <vector>
#include <string>
using namespace std;

Map::Map () {
	loaded = false;
}

void Map::load ( string mapFile ) {
	if ( loaded )
		fatErrMsg ( "Attempt to reload map from " + mapFile + "." );

	FileIn mapIn ( mapFile );

	setFoc ( 0, 0 );

	string line;
	line = mapIn . getLine ();
	int r = atoi ( line . substr ( 0, line . find ( ' ' ) ) . c_str () );
	line = line . substr ( line . find ( ' ' ) + 1, string::npos );
	int g = atoi ( line . substr ( 0, line . find ( ' ' ) ) . c_str () );
	line = line . substr ( line . find ( ' ' ) + 1, string::npos );
	int b = atoi ( line . c_str () );
	bkgdClr = createClr ( r, g, b );

	while ( ! mapIn . eof () ) {
		line = mapIn . getLine ();
		int x = atoi ( line . substr ( 0, line . find ( ' ' ) ) . c_str () );
		line = line . substr ( line . find ( ' ' ) + 1, string::npos );
		int y = atoi ( line . substr ( 0, line . find ( ' ' ) ) . c_str () );
		line = line . substr ( line . find ( ' ' ) + 1, string::npos );
		int imgId = atoi ( line . c_str () );

		MapObj * mapObj = new MapObj ( imgId, x, y );

		mapObjs . push_back ( mapObj );
	}

	loaded = true;
}

void Map::destroy () {
	if ( ! loaded )
		fatErrMsg ( "Attempt to destroy unloaded map." );

	for ( int i = 0; i < ( int ) mapObjs . size (); i++ )
		delete mapObjs [ i ];
	mapObjs . clear ();

	loaded = false;
}

void Map::draw () {
	if ( ! loaded )
		fatErrMsg ( "Attempt to draw unloaded map." );

	clearScreenToClr ( bkgdClr );
	for ( MapObjs::iterator it = mapObjs . begin (); it != mapObjs . end (); it++ )
		(*it) -> draw ( (*it) -> retX1 () - focX, (*it) -> retY1 () - focY);
}

bool Map::emptyPxl ( int x, int y ) {
	const int SCREENX = x - retFocX ();
	const int SCREENY = y - retFocY ();

	if ( SCREENX > 0 && SCREENY > 0 && SCREENX < screenW && SCREENY < screenH )
		if ( clrAtScreenPxl ( SCREENX, SCREENY ) == bkgdClr )
			return true;
	return false;
}

void Map::setFoc ( int focX, int focY ) {
	Map::focX = focX - int ( screenW / 2 );
	Map::focY = focY - int ( screenH / 2 );
}

int pxlsPerMove = INIPXLSPERMOVE;

Map * gameMap = NULL;