#include "obj.h"
#include "map.h"
#include "img.h"
#include "lib.h"
#include "gen.h"
#include "map.h"
#include <string>
#include <map>
using namespace std;

MapObj::MapObj ( int imgId, int x, int y ) {
	sprite = imgList -> retImg ( imgId );

	setX ( x );
	setY ( y );
}

MapObj::MapObj ( Sprite * sprite, int x, int y ) {
	MapObj::sprite = sprite;
	imgList -> addImg ( sprite );

	setX ( x );
	setY ( y );
}

bool MapObj::pxlColl ( Compass dir ) {
	const int MV = pxlsPerMove;

	if ( ! ( dir == n || dir == e || dir == s || dir == w  ) )
		fatErrMsg ( "Function \"pxlColl\" only designed to work with cardinal directions (n, e, s, w)." );

	int FX1, FY1, FX2, FY2; // bounds of the field we need to check to determine if there is a pxlColl
	switch ( dir ) {
		case n:
			FX1 = x1;
			FX2 = x2;
			FY2 = y1 - 1;
			FY1 = FY2 - MV;
			break;
		case e:
			FX1 = x2 + 1;
			FX2 = FX1 + MV;
			FY1 = y1;
			FY2 = y2;
			break;
		case s:
			FX1 = x1;
			FX2 = x2;
			FY1 = y2 + 1;
			FY2 = FY1 + MV;
			break;
		case w:
			FX2 = x1 - 1;
			FX1 = FX2 - MV;
			FY1 = y1;
			FY2 = y2;
			break;
	}

	for ( int pxlRow = FY1; pxlRow <= FY2; pxlRow++ ) {
		for ( int pxlCol = FX1; pxlCol <= FX2; pxlCol++ )
			if ( ! gameMap -> emptyPxl ( pxlCol, pxlRow ) )
				return true;
	}
	return false;
}

bool MapObj::boxColl ( Compass dir, int boxX1, int boxY1, int boxX2, int boxY2 ) {
	const int COLLDEPTH = pxlsPerMove;

	if ( ! ( dir == n || dir == e || dir == s || dir == w  ) )
		fatErrMsg ( "Function \"boxColl\" only designed to work with cardinal directions (n, e, s, w)." );

	int FX1, FY1, FX2, FY2; // bounds of the field we need to check to determine if there is a boxColl
	switch ( dir ) {
		case n:
			FX1 = x1;
			FX2 = x2;
			FY2 = y1 - 1;
			FY1 = FY2 - COLLDEPTH;
			break;
		case e:
			FX1 = x2 + 1;
			FX2 = FX1 + COLLDEPTH;
			FY1 = y1;
			FY2 = y2;
			break;
		case s:
			FX1 = x1;
			FX2 = x2;
			FY1 = y2 + 1;
			FY2 = FY1 + COLLDEPTH;
			break;
		case w:
			FX2 = x1 - 1;
			FX1 = FX2 - COLLDEPTH;
			FY1 = y1;
			FY2 = y2;
			break;
	}
	if ( ( FX1 <= boxX2 && FX2 >= boxX1 ) && ( FY1 <= boxY2 && FY2 >= boxY1 ) )
		return true;
	return false;
}

void MapObj::move ( Compass dir ) {
	const int MV = pxlsPerMove;

	switch ( dir ) {
		case n:
			if ( ! pxlColl ( n ) )
				setY ( y1 - MV );
			break;
		case ne:
			if ( ! pxlColl ( n ) && ! pxlColl ( e ) ) {
				setY ( y1 - MV );
				setX ( x1 + MV );
			} else if ( ! pxlColl ( n ) )
				setY ( y1 - MV );
			else if ( ! pxlColl ( e ) )
				setX ( x1 + MV );
			break;
		case e:
			if ( ! pxlColl ( e ) )
				setX ( x1 + MV );
			break;
		case se:
			if ( ! pxlColl ( s ) && ! pxlColl ( e ) ) {
				setY ( y1 + MV );
				setX ( x1 + MV );
			} else if ( ! pxlColl ( s ) )
				setY ( y1 + MV );
			else if ( ! pxlColl ( e ) )
				setX ( x1 + MV );
			break;
		case s:
			if ( ! pxlColl ( s ) )
				setY ( y1 + MV );
			break;
		case sw:
			if ( ! pxlColl ( s ) && ! pxlColl ( w ) ) {
				setY ( y1 + MV );
				setX ( x1 - MV );
			} else if ( ! pxlColl ( s ) )
				setY ( y1 + MV );
			else if ( ! pxlColl ( w ) )
				setX ( x1 - MV );
			break;
		case w:
			if ( ! pxlColl ( w ) )
				setX ( x1 - MV );
			break;
		case nw:
			if ( ! pxlColl ( n ) && ! pxlColl ( w ) ) {
				setY ( y1 - MV );
				setX ( x1 - MV );
			} else if ( ! pxlColl ( n ) )
				setY ( y1 - MV );
			else if ( ! pxlColl ( w ) )
				setX ( x1 - MV );
			break;
	}
}

void MapObj::draw () {
	sprite -> draw ( x1 - gameMap -> retFocX (), y1 - gameMap -> retFocY () );
}



// FaceObj

FaceObj::FaceObj ( string spriteFile, int x, int y, Compass face )
	: MapObj ( new Sprite ( spriteFile ), x, y ) {
	faces [ face ] = MapObj::sprite;
	FaceObj::face = face;
}

void FaceObj::setFace ( Compass face ) {
	if ( faces . find ( face ) != faces . end () ) {
		const int PREVSPRITEW = sprite -> retW ();
		const int PREVSPRITEH = sprite -> retH ();
		const int NEWSPRITEW = faces [ face ] -> retW ();
		const int NEWSPRITEH = faces [ face ] -> retH ();
		const signed int OFFSETX = ( PREVSPRITEW - NEWSPRITEW ) / 2;
		const signed int OFFSETY = ( PREVSPRITEH - NEWSPRITEH ) / 2;

		MapObj::setX ( x1 + OFFSETX );
		MapObj::setY ( y1 + OFFSETY );
		MapObj::sprite = faces [ face ];
		FaceObj::face = face;
	}
}

void FaceObj::loadFace ( Compass face, string spriteFile ) {
	Sprite * sprite = new Sprite ( spriteFile );
	imgList -> addImg ( sprite );
	faces [ face ] = sprite;
}

void FaceObj::move ( Compass dir ) {
	MapObj::move ( dir );
	setFace ( dir );
}