#include "result.h"
#include "script.h"
#include "combat.h"
#include "pc.h"
#include "event.h"
#include "actor.h"
#include "gui.h"
#include "gen.h"
#include <string>
#include <vector>
#include <cstdlib>
#include <utility>
using namespace std;

const string RESULT_SAY ( "say" );
const string RESULT_SET ( "set" );
const string RESULT_CLEAR ( "clear" );
const string RESULT_SKIPPC ( "skipPc" );
const string RESULT_STATRAISE ( "statRaise" );
const string RESULT_STATLOWER ( "statLower" );
const string RESULT_CHANGEMAP ( "changeMap" );
const string RESULT_FIGHT ( "fight" );



// Set

Set::Set ( Actor * self, string flagName )
: Result ( self ) {
	flagId = eventList . retIndex ( flagName );
}

void Set::run () {
	eventList [ flagId ] = true;
}



// Clear

Clear::Clear ( Actor * self, string flagName )
: Result ( self ) {
	flagId = eventList . retIndex ( flagName );
}

void Clear::run () {
	eventList [ flagId ] = false;
}



// Say

Say::Say ( Actor * self, string line )
: Result ( self ) {
	const string SPEAKER_SELF ( "self" );
	const string SPEAKER_PC ( "pc" );

	const string COLON ( ":\n     " );

	if ( line . find ( '[' ) == string::npos )
		dialogue . push_back ( self -> retName () + COLON + line );
	else {
		pair <string, string> monologue;

		do {
			monologue = stripArgs ( line, '[', ']' );
			if ( monologue . first == SPEAKER_SELF )
				dialogue . push_back ( self -> retName () + COLON + monologue . second );
			else if ( monologue . first == SPEAKER_PC )
				dialogue . push_back ( pc -> retName () + COLON + monologue . second );
			else
				dialogue . push_back ( monologue . first + COLON + monologue . second );
		} while ( ! line . empty () );
	}
}

void Say::run () {
	for ( Dialogue::iterator it = dialogue . begin (); it != dialogue . end (); it ++ ) {
		VoxBox voxBox ( (*it) );
		voxBox . run ();
	}
}



// SkipPc

SkipPc::SkipPc ( Actor * self )
: Result ( self ) {
	;
}

void SkipPc::run () {
	pc -> setSkip ();
}



// StatRaise

StatRaise::StatRaise ( Actor * self, string args )
: Result ( self ) {
	if ( args . find ( ARGDIV ) == string::npos )
		fatErrMsg ( "Not enough arguments in \"" + RESULT_STATRAISE + "\" result with argument string \"" + args + "\"." );

	statName = args . substr ( 0, args . find ( ARGDIV ) );
	statAdd = atoi ( args . substr ( args . find ( ARGDIV ), string::npos ) . c_str () );
}

void StatRaise::run () {
	self -> stats [ statName ] += statAdd;
}



// StatLower

StatLower::StatLower ( Actor * self, string args )
: Result ( self ) {
	if ( args . find ( ARGDIV ) == string::npos )
		fatErrMsg ( "Not enough arguments in \"" + RESULT_STATLOWER + "\" result with argument string \"" + args + "\"." );

	statName = args . substr ( 0, args . find ( ARGDIV ) );
	statSub = atoi ( args . substr ( args . find ( ARGDIV ), string::npos ) . c_str () );
}

void StatLower::run () {
	self -> stats [ statName ] -= statSub;
}



// ChangeMap

ChangeMap::ChangeMap ( Actor * self, string args )
: Result ( self ) {
	newMapName = args . substr ( 0, args . find ( ARGDIV ) );
}

extern string scene;
void ChangeMap::run () {
	scene = newMapName;
}



// Fight

Fight::Fight ( Actor * self )
: Result ( self ) {
	;
}

void Fight::run () {
	combat ( self );
}



// ResultList

string ResultList::stripArg ( string & line ) {
	const Strind argStart = line . find ( ARG1 ) + 1;
	const Strind argEnd = line . find ( ARG2 ) - 1;
	if ( argStart > argEnd ) {
		line = line . substr ( argEnd + 2, string::npos );
		string emptyArg;
		return emptyArg;
	}
	const Strind argLength = argEnd - argStart + 1;
	string lineArg = line . substr ( argStart, argLength );
	line = line . substr ( argEnd + 2, string::npos );
	return lineArg;
}

ResultList::ResultList ( Actor * self, string line ) {
	if ( line . find ( ARG1 ) == string::npos || line . find ( ARG2 ) == string::npos )
		fatErrMsg ( "Actor script improperly formatted at line " + line + ": no argument." );

	while ( line . find ( ARG1 ) != string::npos ) {
		string resultType = line . substr ( 0, line . find ( ARG1 ) );
		if ( resultType == RESULT_SAY ) {
			string speech = stripArg ( line );
			Result * say = new Say ( self, speech );
			results . push_back ( say );
		} else if ( resultType == RESULT_SET ) {
			string flagName = stripArg ( line );
			Result * setFlag = new Set ( self, flagName );
			results . push_back ( setFlag );
		} else if ( resultType == RESULT_CLEAR ) {
			string flagName = stripArg ( line );
			Result * clearFlag = new Clear ( self, flagName );
			results . push_back ( clearFlag );
		} else if ( resultType == RESULT_SKIPPC ) {
			stripArg ( line );
			Result * skipPc = new SkipPc ( self );
			results . push_back ( skipPc );
		} else if ( resultType == RESULT_CHANGEMAP ) {
			string mapName = stripArg ( line );
			Result * changeMap = new ChangeMap ( self, mapName );
			results . push_back ( changeMap );
		} else if ( resultType == RESULT_FIGHT ) {
			stripArg ( line );
			Result * fight = new Fight ( self );
			results . push_back ( fight );
		} else
			fatErrMsg ( "Actor script improperly formatted at line " + line + ": non-existent result used." );
	}
}

ResultList::~ResultList () {
	for ( Results::iterator it = results . begin (); it != results . end (); it++ )
		delete (*it);
}

void ResultList::run () {
	for ( Results::iterator it = results . begin (); it != results . end (); it++ )
		(*it) -> run ();
}